<?php

namespace Chekonline\CashboxModule\Cron;

use Magento\Framework\App\ObjectManager;
use Magento\Sales\Model\Order;
use Chekonline\Cashbox\Commands\ComplexCommand;
use Chekonline\Cashbox\DocumentType;
use Chekonline\Cashbox\Exceptions\ChekonlineCommandException;
use Chekonline\Cashbox\Exceptions\ChekonlineLineException;
use Chekonline\Cashbox\Exceptions\ChekonlineSDKException;
use Chekonline\Cashbox\Line;
use Chekonline\Cashbox\PayAttribute;
use Chekonline\CashboxModule\Helper\ChekonlineCashboxHelper;
use Chekonline\CashboxModule\Helper\Data;
use Chekonline\Cashbox\Api;

class ResendReceiptTask
{
    /** @var ObjectManager  */
    protected $objectManager;

    /** @var \Chekonline\CashboxModule\Helper\ChekonlineCashboxHelper  */
    protected $helper;

    /** @var Api */
    protected $chekonlineApi;

    /**
     * ResendReceiptTask constructor.
     */
    public function __construct()
    {
        $this->objectManager = ObjectManager::getInstance();
        $this->helper = $this->objectManager->create(Data::class);
        $this->chekonlineApi = new Api($this->helper->getGeneralConfig('endpoint'));
    }

    /**
     * Resend receipt
     *
     * @var \Chekonline\CashboxModule\Model\Receipt $receiptModel
     * @var \Chekonline\CashboxModule\Model\ResourceModel\Receipt[] $receiptCollection
     * @var \Chekonline\CashboxModule\Model\ResourceModel\Receipt $receipt
     * @var \Magento\Sales\Model\Order $order
     * @var \Magento\Sales\Api\Data\OrderItemInterface[] $orderItem
     * @var \Magento\Sales\Api\Data\OrderItemInterface $orderItem
     * @var \Chekonline\Cashbox\ChekonlineResponse $responseChekonline
     * @var \Chekonline\Cashbox\ChekonlineRequest $request
     */
    public function execute()
    {
        $this->chekonlineApi->setClientName('Magento');
        $this->chekonlineApi->setClientVer($this->helper->getMagentoVersion());
        if ($this->helper->getGeneralConfig('cert')) {
            $this->chekonlineApi->setCertificate($this->helper->getGeneralConfig('cert'),
                $this->helper->getGeneralConfig('key'),
                $this->helper->getGeneralConfig('cert_password')
            );
        }
        $receiptModel = $this->objectManager->create('\Chekonline\CashboxModule\Model\Receipt');
        /** @var \Chekonline\CashboxModule\Model\Receipt[] $receiptCollection */
        $receiptCollection = $receiptModel->getCollection()
                                ->addFieldToFilter('status', 0)
            //TODO: creditmemo
                                ->addFieldToFilter('type', 0);

        /** @var \Chekonline\CashboxModule\Model\Receipt $receipt */
        foreach ($receiptCollection as $receipt) {
            try {
                $receiptId = strval($receipt->getId());
                $documentType = $receipt->getData('type') == 0 ? DocumentType::DEBIT : DocumentType::REFUND_DEBIT;

                /** @var Order $order */
                $order = $this->objectManager->create(Order::class);
                $order = $order->load(intval($receipt->getData('order_number')));
                $contact = $order->getCustomerEmail();

                $nonCash = round($order->getGrandTotal() * 100);
                $complexCommand = new ComplexCommand($receiptId, $this->helper->getGeneralConfig('client_id'));
                $complexCommand->setPhoneOrEmail($contact)
                    ->setNonCash(array(0, $nonCash, 0))
                    ->setTaxMode(pow(2, $this->helper->getGeneralConfig('tax_mode')))
                    ->setMaxDocumentsInTurn(intval($this->helper->getGeneralConfig('docs_in_turn')))
                    ->setGroup($this->helper->getGeneralConfig('group'))
                    ->setDocumentType($documentType)
                    ->setPassword(intval($this->helper->getGeneralConfig('cashier_password')))
                    ->setFullResponse(false)
                    ->setDevice('auto');


                $orderItems = $order->getItems();


                $productVat = intval($this->helper->getGeneralConfig('product_vat'));

                foreach ($orderItems as $orderItem) {
                    $qty = $orderItem->getQtyOrdered() * 1000;
                    $subtotal = round(($orderItem->getBaseRowTotal() -  $orderItem->getDiscountAmount()) * 100) ;
                    $description = $orderItem->getName();

                    $receiptLine = new Line();
                    $receiptLine->setDescription($description)
                        ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                        ->setSubTotal($subtotal)
                        ->setQty($qty)
                        ->setTaxId($productVat);

                    $complexCommand->addLine($receiptLine);
                }


                $deliveryVat = intval($this->helper->getGeneralConfig('delivery_vat'));
                if ($order->getShippingAmount()) {
                    $qty = 1000;
                    $subtotal  = round(($order->getShippingAmount() - $order->getShippingDiscountAmount()) * 100);
                    $description = 'Доставка';

                    $receiptLine = new Line();
                    $receiptLine->setDescription($description)
                        ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                        ->setQty($qty)
                        ->setSubTotal($subtotal)
                        ->setTaxId($deliveryVat);
                    $complexCommand->addLine($receiptLine);
                }

                $response = $this->chekonlineApi->executeCommand($complexCommand);
                $request = $response->getRequest();

                $receiptStatus = 0;
                if ($this->isReceiptSuccess($response)) {
                    $receiptStatus = 1;
                }

                $receipt->setData('status', $receiptStatus);
                $receipt->save();

                if ($this->helper->getGeneralConfig('log')) {
                    ChekonlineCashboxHelper::writeLog('request', $request);
                    ChekonlineCashboxHelper::writeLog('response', $response);
                    if ($this->isReceiptSuccess($response) == false) {
                        ChekonlineCashboxHelper::writeLog('error', $response);
                    }
                }
            } catch (ChekonlineLineException $exception) {
                $logMessage = "Chekonline line receipt exception:\n\tMessage: {$exception->getMessage()}";
                $logMessage .= "\n\tAt: {$exception->getFile()}\n";
                $logMessage .= "\nLine:\n".var_export($exception->getReceiptLine()->getParam(),true);
                $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
                ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            } catch (ChekonlineCommandException $exception) {
                $logMessage = "Chekonline command exception: \n\tMessage: {$exception->getMessage()}";
                $logMessage .= "\n\tAt: {$exception->getFile()}\n";
                $logMessage .= "\nCommand:".var_export($exception->getCommand()->getParams(),true);
                $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
                ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            } catch (ChekonlineSDKException $exception) {
                $logMessage = "Chekonline exception: \n\tMessage: {$exception->getMessage()}";
                $logMessage .= "\n\tAt: {$exception->getFile()}\n";
                $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
                ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            }
        }
    }

    /**
     * @param \Chekonline\Cashbox\ChekonlineResponse $apiResponse
     * @return bool
     */
    private function isReceiptSuccess($apiResponse)
    {
        if ($apiResponse->getHttpStatusCode() == 200 &&
            $apiResponse->getApiErrorCode() == 0 &&
            $apiResponse->getHttpError() == '') {
            return true;
        } else {
            return false;
        }
    }
}