<?php

namespace Chekonline\CashboxModule\Observer;

use Braintree\Exception;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Event\ObserverInterface;
use Chekonline\Cashbox\Api;
use Chekonline\Cashbox\Commands\ComplexCommand;
use Chekonline\Cashbox\DocumentType;
use Chekonline\Cashbox\Exceptions\ChekonlineCommandException;
use Chekonline\Cashbox\Exceptions\ChekonlineLineException;
use Chekonline\Cashbox\Exceptions\ChekonlineSDKException;
use Chekonline\Cashbox\Line;
use Chekonline\Cashbox\PayAttribute;
use Chekonline\CashboxModule\Helper\Data;
use Chekonline\CashboxModule\Helper\ChekonlineCashboxHelper;

class ChekonlineObserver implements \Magento\Framework\Event\ObserverInterface
{

    /**
     * @var \Magento\Sales\Model\Order
     */
    protected $order;

    /**
     * @var   ObjectManager
     */
    protected $objectManager;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Api
     */
    protected $chekonlineApi;

    public function __construct()
    {
            $this->objectManager = ObjectManager::getInstance();
            $this->helper = $this->objectManager->create(Data::class);

    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return ChekonlineObserver
     */
    public function execute (\Magento\Framework\Event\Observer $observer)
    {
        $this->order = $observer->getData('order');
        $orderStatus = $this->order->getStatus();
        $oldOrderStatus = $this->order->getOrigData('status');
        $orderPayment = $this->order->getPayment()->getMethod();
        if ($this->isStatusForReceipt($orderStatus) &&
            $this->isPaymentMethodForReceipt($orderPayment) &&
            $orderStatus != $oldOrderStatus) {
            try {
                $this->chekonlineApi = new Api($this->helper->getGeneralConfig('endpoint'));
                $this->chekonlineApi->setClientName('Magento');
                $this->chekonlineApi->setClientVer($this->helper->getMagentoVersion());
                if ($this->helper->getGeneralConfig('cert')) {
                    $this->chekonlineApi->setCertificate(
                        $this->helper->getGeneralConfig('cert'),
                        $this->helper->getGeneralConfig('key'),
                        $this->helper->getGeneralConfig('cert_password')
                    );
                }

                /**
                 * validation of parameters api, can cause an exception
                 */
                $this->chekonlineApi->validate();

                /** @var int $orderNumber */
                $orderNumber = $this->order->getId();

                $type = 0;
                $documentType = DocumentType::DEBIT;

                $contact = $this->order->getCustomerEmail();

                /** @var \Chekonline\CashboxModule\Model\Receipt $receiptModel */
                $receiptModel = $this->objectManager->create('\Chekonline\CashboxModule\Model\Receipt');
                /** @var \Chekonline\CashboxModule\Model\ResourceModel\Receipt\Collection $receiptCollection */
                $receiptCollection = $receiptModel->getCollection()
                    ->addFieldToFilter('order_number', $orderNumber)
                    ->addFieldToFilter('type', $type);

                if ($receiptCollection->count() == 0) {
                    $receiptModel->setData('order_number', $orderNumber)
                        ->setData('type', $type)
                        ->setData('status', 0)
                        ->setData('date_of_printing', date("Y-m-d H:i:s"));
                    $receiptModel->save();
                } else {
                    $receiptModel = $receiptCollection->getFirstItem();
                }
                $receiptId = strval($receiptModel->getId());

                $complexCommand = new ComplexCommand($receiptId);
                $complexCommand->setClientId($this->helper->getGeneralConfig('client_id'));

                $complexCommand->setPhoneOrEmail($contact)
                    ->setTaxMode(pow(2, $this->helper->getGeneralConfig('tax_mode')))
                    ->setMaxDocumentsInTurn(intval($this->helper->getGeneralConfig('docs_in_turn')))
                    ->setGroup($this->helper->getGeneralConfig('group'))
                    ->setDocumentType($documentType)
                    ->setPassword(intval($this->helper->getGeneralConfig('cashier_password')))
                    ->setFullResponse(false)
                    ->setDevice('auto');

                /** @var \Magento\Sales\Api\Data\OrderItemInterface[] $orderItem */
                $orderItems = $this->order->getAllVisibleItems();

                $productVat = intval($this->helper->getGeneralConfig('product_vat'));
                /** @var \Magento\Sales\Api\Data\OrderItemInterface $orderItem */
                foreach ($orderItems as $orderItem) {

                    $qty = $orderItem->getQtyOrdered() * 1000;

                    $subtotal = round(($orderItem->getBaseRowTotal() - $orderItem->getDiscountAmount()) * 100);

                    if ($subtotal <= 0) {
                        continue;
                    }

                    $description = $orderItem->getName();

                    $receiptLine = new Line();

                    $receiptLine->setDescription($description)
                        ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                        ->setSubTotal($subtotal)
                        ->setQty($qty)
                        ->setTaxId($productVat);
                    /**
                     * validation of parameters line before adding to the receipt, can cause an exception
                     */
                    $receiptLine->validate();
                    $complexCommand->addLine($receiptLine);
                }

                $deliveryVat = intval($this->helper->getGeneralConfig('delivery_vat'));
                if ($this->order->getShippingAmount()) {
                    $qty = 1000;
                    $subtotal = round(($this->order->getShippingAmount() - $this->order->getShippingDiscountAmount()) * 100);
                    $description = 'Доставка';

                    $receiptLine = new Line();

                    $receiptLine->setDescription($description)
                        ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                        ->setSubTotal($subtotal)
                        ->setQty($qty)
                        ->setTaxId($deliveryVat);
                    /**
                     * validation of parameters line before adding to the receipt, can cause an exception
                     */
                    $receiptLine->validate();
                    $complexCommand->addLine($receiptLine);
                }


                $orderTotal =  round($this->order->getGrandTotal() * 100);
                $complexCommand->normalizeBySubTotal($orderTotal);
                $complexCommand->setNonCash(array(0, $orderTotal, 0));
                /**
                 * validation of parameters complex command before executing, can cause an exception
                 */
                $complexCommand->validate();
                /**
                 * validation of parameters api before call, can cause an exception
                 */
                $this->chekonlineApi->validate();
                $response = $this->chekonlineApi->executeCommand($complexCommand);
                $request = $response->getRequest();

                $receiptStatus = 0;
                if ($this->isReceiptSuccess($response)) {
                    $receiptStatus = 1;
                }

                $receiptModel->setData('status', $receiptStatus);
                $receiptModel->save();

                if ($this->helper->getGeneralConfig('log')) {
                    ChekonlineCashboxHelper::writeLog('request', $request);
                    ChekonlineCashboxHelper::writeLog('response', $response);
                }
                if ($this->isReceiptSuccess($response) == false) {
                    ChekonlineCashboxHelper::writeLog('error', $response);
                }

            } catch (ChekonlineLineException $exception) {
                $logMessage = "Chekonline line receipt exception:\n\tMessage: {$exception->getMessage()}";
                $logMessage .= "\n\tAt: {$exception->getFile()}\n";
                $logMessage .= "\nLine:\n" . var_export($exception->getReceiptLine()->getParam(), true);
                $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
                ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            } catch (ChekonlineCommandException $exception) {
                $logMessage = "Chekonline command exception: \n\tMessage: {$exception->getMessage()}";
                $logMessage .= "\n\tAt: {$exception->getFile()}\n";
                $logMessage .= "\nCommand:" . var_export($exception->getCommand()->getParams(), true);
                $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
                ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            } catch (ChekonlineSDKException $exception) {
                $logMessage = "Chekonline exception: \n\tMessage: {$exception->getMessage()}";
                $logMessage .= "\n\tAt: {$exception->getFile()}\n";
                $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
                ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            }
        }
        return $this;
    }



    /**
     * @param int $status
     * @return bool
     */
    private function isStatusForReceipt($status)
    {
        $statuses_paid = explode(',', $this->helper->getGeneralConfig('status_paid'));

        $statuses = array_merge($statuses_paid);

        return in_array($status, $statuses);
    }

    /**
     * @param string $paymentMethod
     * @return bool
     */
    private function isPaymentMethodForReceipt($paymentMethod)
    {
        $paymentMethods = explode(',', $this->helper->getGeneralConfig('payment'));

        return in_array($paymentMethod, $paymentMethods);
    }

    /**
     * @param \Chekonline\Cashbox\ChekonlineResponse $apiResponse
     * @return bool
     */
    private function isReceiptSuccess($apiResponse)
    {
        if ($apiResponse->getHttpStatusCode() == 200 &&
            $apiResponse->getApiErrorCode() == 0 &&
            $apiResponse->getHttpError() == '') {
            return true;
        } else {
            return false;
        }
    }
}