<?php

namespace Chekonline\CashboxModule\Observer;


use Magento\Framework\App\ObjectManager;
use Magento\Framework\Event\Observer;
use Chekonline\Cashbox\Api;
use Chekonline\Cashbox\Commands\ComplexCommand;
use Chekonline\Cashbox\DocumentType;
use Chekonline\Cashbox\Exceptions\ChekonlineCommandException;
use Chekonline\Cashbox\Exceptions\ChekonlineLineException;
use Chekonline\Cashbox\Exceptions\ChekonlineSDKException;
use Chekonline\Cashbox\Line;
use Chekonline\Cashbox\PayAttribute;
use Chekonline\CashboxModule\Helper\Data;
use Chekonline\CashboxModule\Helper\ChekonlineCashboxHelper;

class ChekonlineObserverCreditmemo implements  \Magento\Framework\Event\ObserverInterface
{

    /**
     * @var Api
     */
    protected $chekonlineApi;

    /**
     * @var   ObjectManager
     */
    protected $objectManager;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * ChekonlineObserverCreditmemo constructor.
     */
    public function __construct()
    {
        $this->objectManager = ObjectManager::getInstance();
        $this->helper = $this->objectManager->create(Data::class);

    }

    /**
     * @param Observer $observer
     * @return $this
     */
    public function execute (Observer $observer)
    {
        /** @var \Magento\Sales\Model\Order\Creditmemo $creditmemo */
        $creditmemo = $observer->getEvent()->getCreditmemo();
        $order = $creditmemo->getOrder();

        try {
            $this->chekonlineApi = new Api($this->helper->getGeneralConfig('endpoint'));
            $this->chekonlineApi->setClientName('Magento');
            $this->chekonlineApi->setClientVer($this->helper->getMagentoVersion());
            if ($this->helper->getGeneralConfig('cert')) {
                $this->chekonlineApi->setCertificate($this->helper->getGeneralConfig('cert'),
                    $this->helper->getGeneralConfig('key'),
                    $this->helper->getGeneralConfig('cert_password')
                );
            }

            /**
             * validation of parameters api, can cause an exception
             */
            $this->chekonlineApi->validate();

            /** @var int $memoNumber */
            $memoNumber = $creditmemo->getId();

            $type = 1;

            $documentType = DocumentType::REFUND_DEBIT;

            $contact = $order->getCustomerEmail();

            /** @var \Chekonline\CashboxModule\Model\Receipt $receiptModel */
            $receiptModel = $this->objectManager->create('\Chekonline\CashboxModule\Model\Receipt');
            /** @var \Chekonline\CashboxModule\Model\ResourceModel\Receipt\Collection $receiptCollection */
            $receiptCollection = $receiptModel->getCollection()
                ->addFieldToFilter('order_number', $memoNumber)
                ->addFieldToFilter('type', $type);

            if ($receiptCollection->count() == 0) {
                $receiptModel->setData('order_number', $memoNumber)
                    ->setData('type', $type)
                    ->setData('status', 0)
                    ->setData('date_of_printing', date("Y-m-d H:i:s"));
                $receiptModel->save();
            } else {
                $receiptModel = $receiptCollection->getFirstItem();
            }
            $receiptId = strval($receiptModel->getId());

            $complexCommand = new ComplexCommand($receiptId);
            $complexCommand->setClientId($this->helper->getGeneralConfig('client_id'));

            $complexCommand->setPhoneOrEmail($contact)
                ->setTaxMode(pow(2, $this->helper->getGeneralConfig('tax_mode')))
                ->setMaxDocumentsInTurn(intval($this->helper->getGeneralConfig('docs_in_turn')))
                ->setGroup($this->helper->getGeneralConfig('group'))
                ->setDocumentType($documentType)
                ->setPassword(intval($this->helper->getGeneralConfig('cashier_password')))
                ->setFullResponse(false)
                ->setDevice('auto');

            $creditmemoItems = $creditmemo->getItems();

            $productVat = intval($this->helper->getGeneralConfig('product_vat'));

            foreach ($creditmemoItems as $creditmemoItem) {
                $qty = $creditmemoItem->getQty() * 1000;
                $subtotal = round(($creditmemoItem->getBaseRowTotal() - $creditmemoItem->getDiscountAmount()) * 100);
                if ($subtotal <=0) {
                    continue;
                }
                $description = $creditmemoItem->getName();

                $receiptLine = new Line();
                $receiptLine->setDescription($description)
                    ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                    ->setSubTotal($subtotal)
                    ->setQty($qty)
                    ->setTaxId($productVat);
                /**
                 * validation of parameters line before adding to the receipt, can cause an exception
                 */
                $receiptLine->validate();
                $complexCommand->addLine($receiptLine);
            }

            $deliveryVat = intval($this->helper->getGeneralConfig('delivery_vat'));
            if ($creditmemo->getShippingAmount()) {
                $qty = 1000;
                $subtotal = round(($creditmemo->getData('shipping_amount')) * 100);
                $description = 'Доставка';

                $receiptLine = new Line();
                $receiptLine->setDescription($description)
                    ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                    ->setSubTotal($subtotal)
                    ->setQty($qty)
                    ->setTaxId($deliveryVat);
                /**
                 * validation of parameters line before adding to the receipt, can cause an exception
                 */
                $receiptLine->validate();
                $complexCommand->addLine($receiptLine);
            }
            $orderTotal =  round($creditmemo->getGrandTotal() * 100);
            $complexCommand->normalizeBySubTotal($orderTotal);
            $complexCommand->setNonCash(array(0, $orderTotal, 0));

            /**
             * validation of parameters complex command before executing, can cause an exception
             */
            $complexCommand->validate();
            /**
             * validation of parameters api before call, can cause an exception
             */
            $this->chekonlineApi->validate();
            $response = $this->chekonlineApi->executeCommand($complexCommand);
            $request = $response->getRequest();

            $receiptStatus = 0;
            if ($this->isReceiptSuccess($response)) {
                $receiptStatus = 1;
            }

            $receiptModel->setData('status', $receiptStatus);
            $receiptModel->save();

            if ($this->helper->getGeneralConfig('log')) {
                ChekonlineCashboxHelper::writeLog('request', $request);
                ChekonlineCashboxHelper::writeLog('response', $response);
            }
            if ($this->isReceiptSuccess($response) == false) {
                ChekonlineCashboxHelper::writeLog('error', $response);
            }

        } catch (ChekonlineLineException $exception) {
            $logMessage = "Chekonline line receipt exception:\n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nLine:\n" . var_export($exception->getReceiptLine()->getParam(), true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineCashboxHelper::writeLog('fail', $logMessage);
        } catch (ChekonlineCommandException $exception) {
            $logMessage = "Chekonline command exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nCommand:" . var_export($exception->getCommand()->getParams(), true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineCashboxHelper::writeLog('fail', $logMessage);
        } catch (ChekonlineSDKException $exception) {
            $logMessage = "Chekonline exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineCashboxHelper::writeLog('fail', $logMessage);
        }

        return $this;
    }

    /**
     * @param \Chekonline\Cashbox\ChekonlineResponse $apiResponse
     * @return bool
     */
    private function isReceiptSuccess($apiResponse)
    {
        if ($apiResponse->getHttpStatusCode() == 200 &&
            $apiResponse->getApiErrorCode() == 0 &&
            $apiResponse->getHttpError() == '') {
            return true;
        } else {
            return false;
        }
    }
}